// Power supply control API
// Copyright (C) 2007 H.Momose
// ChangeLog
//    Ver 1.0.1.0	t@[EFAXVvÕCv̂߁At@[XV
//					ƋʂƂȂ֐serial.cֈڐ݂B

#include <stdio.h>
#include "serial.h"
#include "PsuControl.h"

#define TX_INTERVAL		10		// ԑMC^[o

// Internal static global
static commio_t *comm;

/*****************************************************************************
 * Function : psu_init 
 * Argument : port <= VA|[g (ex. "com1")
 * Return   : PSU_SUCCESS
 *			: PSU_INVALID_COM
 *			: PSU_INVALID_DEVICE
 *			: PSU_ERROR_COM
 * Remarks  : 
 *****************************************************************************/
int psu_init(const char *port)
{
	char	buff[256];
	int		i, len, read_len, rc;

	// VA|[gI[v
	comm = comm_open(port, 9600);
	if(!comm){
		rc = PSU_INVALID_COM;
		goto fail;
	}

	// foCX̏Ԃ𓯊(SR}h̃f[^҂ȂǂēƂ)
	// X 𑗐MāANG\r\n߂ΓOK
	for(i =0; i < 10; i++){
		len = comm_write(comm, "X", 1);
		if(len != 1){
			rc = PSU_ERROR_COM;
			goto fail;
		}
	
		Sleep(30);
		for(read_len = 0; read_len < sizeof(buff) -1; ){
			len = comm_read(comm, buff + read_len, sizeof(buff) - read_len -1);
			if(len < 0){
				rc = PSU_ERROR_COM;
				goto fail;
		
			} else if(len == 0){
				break;

			} else {
				read_len += len;
				buff[read_len] = 0;		
				if(!strcmp(buff, "NG\r\n")){
					// OK
					goto sync_ok;
				}
			}
		}
	}
	// s
	rc = PSU_INVALID_DEVICE;
	goto fail;

sync_ok:
	// I R}h𑗐MăfoCXmF
	len = comm_setline(comm, "I", 1, TX_INTERVAL);
	if(len != 1){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// X|XM
	len = comm_getline(comm, buff, sizeof(buff) -1);
	if(len < 0){
		rc = PSU_ERROR_COM;
		goto fail;
	}
	buff[len] = 0;
	
	if(strcmp(buff, "USBPSU\r\n")){
		rc = PSU_INVALID_DEVICE;
		goto fail;
	}

	// 
	return(PSU_SUCCESS);

fail:
	// s
	if(comm){
		comm_close(comm);
		comm = NULL;
	}
	return(rc);
}


/*****************************************************************************
 * Function : psu_fini 
 * Argument : Ȃ
 * Return   : Ȃ
 * Remarks  : 
 *****************************************************************************/
void psu_fini()
{
	if(comm){
		comm_close(comm);
		comm = NULL;
	}
}


/*****************************************************************************
 * Function : psu_getVersion
 * Argument : version => o[W
 * Return	: PSU_SUCCESS
 *			: PSU_NOT_INITIALIZED
 *			: PSU_ERROR_COM
 *			: PSU_INVALID_DEVICE
 * Remarks  : 
 *****************************************************************************/
int psu_getVersion(char *version)
{
	int		rc, len;
	char	buff[256];

	if(!comm){
		rc = PSU_NOT_INITIALIZED;
		goto fail;
	}
	
	// V R}h𑗐M
	len = comm_setline(comm, "V", 1, TX_INTERVAL);
	if(len != 1){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// X|XM
	len = comm_getline(comm, buff, sizeof(buff) -1);
	if(len < 0){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// ȂƂA3oCgȏ゠͂
	if(len < 3){
		rc = PSU_INVALID_DEVICE;
		goto fail;
	}

	buff[len] = 0;

	// 2oCg\r\n̂͂Ȃ̂ŁARs[Ȃ
	strncpy(version, buff, len -2);
	version[len -2] = 0;

	return(PSU_SUCCESS);

fail:
	return(rc);
}
	

/*****************************************************************************
 * Function : psu_getStatus
 * Argument : psustat => dXe[^X
 * Return	: PSU_SUCCESS
 *			: PSU_NOT_INITIALIZED
 *			: PSU_ERROR_COM
 *			: PSU_INVALID_DEVICE
 * Remarks  : 
 *****************************************************************************/
int psu_getStatus(struct psustat_st *psustat)
{
	int		rc, len, value;
	char	buff[256];

	if(!comm){
		rc = PSU_NOT_INITIALIZED;
		goto fail;
	}

	// M R}h𑗐M
	len = comm_setline(comm, "M", 1, TX_INTERVAL);
	if(len != 1){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// X|XM
	len = comm_getline(comm, buff, sizeof(buff) -1);
	if(len < 0){
		rc = PSU_ERROR_COM;
		goto fail;
	}
	
	// X|X11oCĝ͂ł
	if(len != 11){
		rc = PSU_INVALID_DEVICE;
		goto fail;
	}

	// d\[X
	if(buff[0] == 'U'){
		// USBoXp[
		psustat->source = SOURCE_USB;
	} else if(buff[0] == 'A'){
		// ACA_v^d(VCC OK)
		psustat->source = SOURCE_ADAPTER;
	} else { // if(buff[0] == 'N'){
		// ACA_v^ڑĂ邪AVCCĂ
		psustat->source = SOURCE_NONE;
	}

	// o̓XCb`
	if(buff[1] == 'N'){
		psustat->output = TRUE;
	} else {
		psustat->output = FALSE;
	}

	// (CV/CC)
	if(buff[2] ==  'C'){
		// d
		psustat->ccmode = TRUE;
	} else { // if(buff[2] == 'V'){
		// d
		psustat->ccmode = FALSE;
	}
	
	// dݒl
	value = hex2int(buff[3]) * 256 + hex2int(buff[4]) * 16 + hex2int(buff[5]);
	psustat->voltage = (double)value * (double)VSTEP;

	// dݒl
	value = hex2int(buff[6]) * 256 + hex2int(buff[7]) * 16 + hex2int(buff[8]);
	psustat->current = (double)value * (double)ASTEP;

	return(PSU_SUCCESS);

fail:
	return(rc);
}


/*****************************************************************************
 * Function : psu_getLimit
 * Argument : voltage => ݂̐ݒd
 *			: current => ݂̐ݒd
 * Return	: PSU_SUCCESS
 *			: PSU_NOT_INITIALIZED
 *			: PSU_ERROR_COM
 *			: PSU_INVALID_DEVICE
 * Remarks  : 
 *****************************************************************************/
int psu_getLimit(double *voltage, double *current)
{
	char	buff[256];
	int		rc, len, value;

	if(!comm){
		rc = PSU_NOT_INITIALIZED;
		goto fail;
	}

	// L R}hM
	len = comm_setline(comm, "L", 1, TX_INTERVAL);
	if(len != 1){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// X|XM
	len = comm_getline(comm, buff, sizeof(buff) -1);
	if(len < 0){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// X|X8oCĝ͂ł
	if(len != 8){
		rc = PSU_INVALID_DEVICE;
		goto fail;
	}

	buff[len] = 0;

	// d
	if(voltage){
		value = hex2int(buff[0]) * 256 + hex2int(buff[1]) * 16 + hex2int(buff[2]);
		*voltage = (double)value * (double)VSTEP;
	}

	// d
	if(current){
		value = hex2int(buff[3]) * 256 + hex2int(buff[4]) * 16 + hex2int(buff[5]);
		*current = (double)value * (double)ASTEP;
	}

	return(PSU_SUCCESS);

fail:
	return(FALSE);
}


/*****************************************************************************
 * Function : psu_setLimit
 * Argument : voltage <= ݒd
 *			: current <= ݒd
 * Return	: PSU_SUCCESS
 *			: PSU_NOT_INITIALIZED
 *			: PSU_ERROR_COM
 *			: PSU_INVALID_DEVICE
 * Remarks  : 
 *****************************************************************************/
int psu_setLimit(double voltage, double current)
{
	char	buff[256];
	int		rc, len, vvalue, avalue;

	if(!comm){
		rc = PSU_NOT_INITIALIZED;
		goto fail;
	}

	vvalue = (int) (voltage / VSTEP);
	avalue = (int) (current / ASTEP);

	// S R}h쐬
	sprintf(buff, "S%c%c%c%c%c%c",
		int2hex(0x0f & (vvalue >> 8)),
		int2hex(0x0f & (vvalue >> 4)),
		int2hex(0x0f &  vvalue),
		int2hex(0x0f & (avalue >> 8)),
		int2hex(0x0f & (avalue >> 4)),
		int2hex(0x0f &  avalue));

	// S R}hM
	len = comm_setline(comm, buff, 7, TX_INTERVAL);
	if(len != 7){
		rc = PSU_ERROR_COM;
		goto fail;
	}
	
	// X|XM
	len = comm_getline(comm, buff, sizeof(buff) -1);
	if(len < 0){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	buff[len] = 0;
	// X|X OK\r\n ̂͂ł
	if(len != 4 || strcmp(buff, "OK\r\n")){
		rc = PSU_INVALID_DEVICE;
		goto fail;
	}

	return(PSU_SUCCESS);

fail:
	return(rc);
}


/*****************************************************************************
 * Function : psu_on
 * Argument : none
 * Return	: PSU_SUCCESS
 *			: PSU_NOT_INITIALIZED
 *			: PSU_ERROR_COM
 *			: PSU_INVALID_DEVICE
 * Remarks  : 
 *****************************************************************************/
int psu_on()
{
	char	buff[256];
	int		rc, len;

	if(!comm){
		rc = PSU_NOT_INITIALIZED;
		goto fail;
	}

	// N R}hM
	len = comm_setline(comm, "N", 1, TX_INTERVAL);
	if(len != 1){
		rc = PSU_ERROR_COM;
		goto fail;
	}
	
	// X|XM
	len = comm_getline(comm, buff, sizeof(buff) -1);
	if(len < 0){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// X|X OK\r\n ̂͂ł
	buff[len]=0;
	if(len != 4 || strcmp(buff, "OK\r\n")){
		rc = PSU_INVALID_DEVICE;
		goto fail;
	}

	return(PSU_SUCCESS);

fail:
	return(rc);
}


/*****************************************************************************
 * Function : psu_off
 * Argument : none
 * Return	: PSU_SUCCESS
 *			: PSU_NOT_INITIALIZED
 *			: PSU_ERROR_COM
 *			: PSU_INVALID_DEVICE
 * Remarks  : 
 *****************************************************************************/
int psu_off()
{
	char	buff[256];
	int		rc, len;

	if(!comm){
		rc = PSU_NOT_INITIALIZED;
		goto fail;
	}

	// F R}hM
	len = comm_setline(comm, "F", 1, TX_INTERVAL);
	if(len != 1){
		rc = PSU_ERROR_COM;
		goto fail;
	}
	
	// X|XM
	len = comm_getline(comm, buff, sizeof(buff) -1);
	if(len < 0){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// X|X OK\r\n ̂͂ł
	buff[len]=0;
	if(len != 4 || strcmp(buff, "OK\r\n")){
		rc = PSU_INVALID_DEVICE;
		goto fail;
	}

	return(PSU_SUCCESS);

fail:
	return(rc);
}


