// Power supply firmware updater API
// Copyright (C) 2007 H.Momose
// ChangeLog
//    Ver 1.0.1.0	t@[EFAXVvÕCv̂ߐV݂B

#include <stdio.h>
#include "serial.h"
#include "PsuControl.h"
#include "PsuFirmup.h"

#define	TX_INTERVAL		5		// ԑMC^[o

// Internal static global
static commio_t *comm;

/*****************************************************************************
 * Function : psu_firmup_init 
 * Argument : port <= VA|[g (ex. "com1")
 * Return   : PSU_SUCCESS
 *			: PSU_INVALID_COM
 *			: PSU_INVALID_DEVICE
 *			: PSU_ERROR_COM
 * Remarks  : 
 *****************************************************************************/
int psu_firmup_init(const char *port)
{
	char	buff[256];
	int		i, len, read_len, rc;

	// VA|[gI[v
	comm = comm_open(port, 9600);
	if(!comm){
		rc = PSU_INVALID_COM;
		goto fail;
	}

	// X 𑗐MāANG\r\n߂ΓOK
	for(i =0; i < 10; i++){
		len = comm_write(comm, "X", 1);
		if(len != 1){
			rc = PSU_ERROR_COM;
			goto fail;
		}
	
		Sleep(30);
		for(read_len = 0; read_len < sizeof(buff) -1; ){
			len = comm_read(comm, buff + read_len, sizeof(buff) - read_len -1);
			if(len < 0){
				rc = PSU_ERROR_COM;
				goto fail;
		
			} else if(len == 0){
				break;

			} else {
				read_len += len;
				buff[read_len] = 0;		
				if(!strcmp(buff, "NG\r\n")){
					// OK
					goto sync_ok;
				}
			}
		}
	}
	// s
	rc = PSU_INVALID_DEVICE;
	goto fail;

sync_ok:
	// I R}h𑗐MăfoCXmF
	len = comm_setline(comm, "I", 1, TX_INTERVAL);
	if(len != 1){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// X|XM
	len = comm_getline(comm, buff, sizeof(buff) -1);
	if(len < 0){
		rc = PSU_ERROR_COM;
		goto fail;
	}
	buff[len] = 0;
	
	if(strcmp(buff, "UPDATER\r\n")){
		rc = PSU_INVALID_DEVICE;
		goto fail;
	}

	// 
	return(PSU_SUCCESS);

fail:
	// s
	if(comm){
		comm_close(comm);
		comm = NULL;
	}
	return(rc);
}


/*****************************************************************************
 * Function : psu_firmup_fini 
 * Argument : Ȃ
 * Return   : Ȃ
 * Remarks  : 
 *****************************************************************************/
void psu_firmup_fini()
{
	if(comm){
		comm_close(comm);
		comm = NULL;
	}
}


/*****************************************************************************
 * Function : psu_firmup_erase
 * Argument : none
 * Return	: PSU_SUCCESS
 *			: PSU_NOT_INITIALIZED
 *			: PSU_ERROR_COM
 *			: PSU_INVALID_DEVICE
 * Remarks  : 
 *****************************************************************************/
int psu_firmup_erase()
{
	char	buff[256];
	int		rc, len;

	if(!comm){
		rc = PSU_NOT_INITIALIZED;
		goto fail;
	}

	// E R}hM
	len = comm_setline(comm, "E", 1, TX_INTERVAL);
	if(len != 1){
		rc = PSU_ERROR_COM;
		goto fail;
	}
	
	// X|XM
	len = comm_getline(comm, buff, sizeof(buff) -1);
	if(len < 0){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// X|X OK\r\n ̂͂ł
	buff[len]=0;
	if(len != 4 || strcmp(buff, "OK\r\n")){
		rc = PSU_INVALID_DEVICE;
		goto fail;
	}

	return(PSU_SUCCESS);

fail:
	return(rc);
}
	

/*****************************************************************************
 * Function : psu_firmup_write
 * Argument : data <= ]t@[EFA
 *			: 
 * Return	: PSU_SUCCESS
 *			: PSU_NOT_INITIALIZED
 *			: PSU_ERROR_COM
 *			: PSU_INVALID_DEVICE
 * Remarks  : data8oCgł邱
 *****************************************************************************/
int psu_firmup_write(const unsigned char *data)
{
	char	buff[256];
	int		i, rc, len;

	if(!comm){
		rc = PSU_NOT_INITIALIZED;
		goto fail;
	}

	// W R}h쐬
	buff[0] = (char)'W';
	for(i = 0; i < 8; i++){
		buff[i*2 +1] = int2hex(0x0f & data[i] >> 4);
		buff[i*2 +2] = int2hex(0x0f & data[i]);
	}

	// W R}hM
	len = comm_setline(comm, buff, 17, TX_INTERVAL);
	if(len != 17){
		rc = PSU_ERROR_COM;
		goto fail;
	}
	
	// X|XM
	len = comm_getline(comm, buff, sizeof(buff) -1);
	if(len < 0){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	buff[len] = 0;
	// X|X OK\r\n ̂͂ł
	if(len != 4 || strcmp(buff, "OK\r\n")){
		rc = PSU_INVALID_DEVICE;
		goto fail;
	}

	return(PSU_SUCCESS);

fail:
	return(rc);
}


/*****************************************************************************
 * Function : psu_firmup_quit
 * Argument : none
 * Return	: PSU_SUCCESS
 *			: PSU_NOT_INITIALIZED
 *			: PSU_ERROR_COM
 *			: PSU_INVALID_DEVICE
 * Remarks  : 
 *****************************************************************************/
int psu_firmup_quit()
{
	int		rc, len;

	if(!comm){
		rc = PSU_NOT_INITIALIZED;
		goto fail;
	}

	// Q R}hM
	len = comm_setline(comm, "Q", 1, TX_INTERVAL);
	if(len != 1){
		rc = PSU_ERROR_COM;
		goto fail;
	}

	// ̃R}h̓X|XȂ
	return(PSU_SUCCESS);

fail:
	return(rc);
}
